@@ -123,7 +123,7 @@ use {super::fs::MemfdFlags, super::io::UserFaultFdFlags};
 #[cfg(not(target_os = "wasi"))]
 use {
     super::io::{DupFlags, MapFlags, ProtFlags, Termios, Winsize},
-    super::time::ClockId,
+    super::time::{ClockId, DynamicClockId},
 };
 
 pub(crate) fn read(fd: BorrowedFd<'_>, buf: &mut [u8]) -> io::Result<usize> {
@@ -579,36 +579,33 @@ pub(crate) fn copy_file_range(
     target_os = "redox"
 )))]
 pub(crate) fn fadvise(fd: BorrowedFd<'_>, offset: u64, len: u64, advice: Advice) -> io::Result<()> {
-    if let (Ok(offset), Ok(len)) = (offset.try_into(), len.try_into()) {
-        let offset: i64 = offset;
-        let len: i64 = len;
+    let offset = offset as i64;
+    let len = len as i64;
 
-        // FreeBSD returns `EINVAL` on overflow; emulate the POSIX behavior.
-        #[cfg(target_os = "freebsd")]
-        let len = if offset.checked_add(len).is_none() {
-            i64::MAX - offset
-        } else {
-            len
-        };
+    // FreeBSD returns `EINVAL` on invalid offsets; emulate the POSIX behavior.
+    #[cfg(target_os = "freebsd")]
+    let offset = if (offset as i64) < 0 {
+        i64::MAX
+    } else {
+        offset
+    };
 
-        let err =
-            unsafe { libc_posix_fadvise(borrowed_fd(fd), offset, len, advice as libc::c_int) };
+    // FreeBSD returns `EINVAL` on overflow; emulate the POSIX behavior.
+    #[cfg(target_os = "freebsd")]
+    let len = if len > 0 && offset.checked_add(len).is_none() {
+        i64::MAX - offset
+    } else {
+        len
+    };
 
-        // `posix_fadvise` returns its error status rather than using `errno`.
-        return if err == 0 {
-            Ok(())
-        } else {
-            Err(io::Error(err))
-        };
-    }
+    let err = unsafe { libc_posix_fadvise(borrowed_fd(fd), offset, len, advice as libc::c_int) };
 
-    if (len as i64) < 0 {
-        return Err(io::Error::INVAL);
-    }
-
-    // If the offset or length can't be converted, ignore the advice, as it
-    // isn't likely to be useful in that case.
-    Ok(())
+    // `posix_fadvise` returns its error status rather than using `errno`.
+    return if err == 0 {
+        Ok(())
+    } else {
+        Err(io::Error(err))
+    };
 }
 
 pub(crate) fn fcntl_getfd(fd: BorrowedFd<'_>) -> io::Result<FdFlags> {
@@ -1094,6 +1091,10 @@ pub(crate) fn ioctl_tiocgwinsz(fd: BorrowedFd) -> io::Result<Winsize> {
     }
 }
 
+/// # Safety
+///
+/// `mmap` is primarily unsafe due to the `addr` parameter, as anything working
+/// with memory pointed to by raw pointers is unsafe.
 #[cfg(not(target_os = "wasi"))]
 pub(crate) unsafe fn mmap(
     ptr: *mut c_void,
@@ -1520,7 +1521,6 @@ pub(crate) fn clock_getres(id: ClockId) -> Timespec {
     }
 }
 
-/// `clock_gettime(id)`
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[must_use]
@@ -1540,7 +1540,50 @@ pub(crate) fn clock_gettime(id: ClockId) -> Timespec {
     }
 }
 
-/// `clock_nanosleep(id, 0, request, remain)`
+#[cfg(not(target_os = "wasi"))]
+#[inline]
+#[must_use]
+pub(crate) fn clock_gettime_dynamic(id: DynamicClockId) -> io::Result<Timespec> {
+    let mut timespec = MaybeUninit::<Timespec>::uninit();
+    unsafe {
+        let id: libc::clockid_t = match id {
+            DynamicClockId::Known(id) => id as libc::clockid_t,
+
+            #[cfg(any(target_os = "android", target_os = "linux"))]
+            DynamicClockId::Dynamic(fd) => {
+                use crate::io::AsRawFd;
+                const CLOCKFD: i32 = 3;
+                (!fd.as_raw_fd() << 3) | CLOCKFD
+            }
+
+            #[cfg(not(any(target_os = "android", target_os = "linux")))]
+            DynamicClockId::Dynamic(_fd) => {
+                // Dynamic clocks are not supported on this platform.
+                return Err(io::Error::INVAL);
+            }
+
+            #[cfg(any(target_os = "android", target_os = "linux"))]
+            DynamicClockId::RealtimeAlarm => libc::CLOCK_REALTIME_ALARM,
+
+            #[cfg(any(target_os = "android", target_os = "linux"))]
+            DynamicClockId::Tai => libc::CLOCK_TAI,
+
+            #[cfg(any(target_os = "android", target_os = "linux"))]
+            DynamicClockId::Boottime => libc::CLOCK_BOOTTIME,
+
+            #[cfg(any(target_os = "android", target_os = "linux"))]
+            DynamicClockId::BoottimeAlarm => libc::CLOCK_BOOTTIME_ALARM,
+        };
+
+        ret(libc::clock_gettime(
+            id as libc::clockid_t,
+            timespec.as_mut_ptr(),
+        ))?;
+
+        Ok(timespec.assume_init())
+    }
+}
+
 #[cfg(not(any(
     target_os = "macos",
     target_os = "ios",
@@ -1567,7 +1610,6 @@ pub(crate) fn clock_nanosleep_relative(id: ClockId, request: &Timespec) -> Nanos
     }
 }
 
-/// `clock_nanosleep(id, TIMER_ABSTIME, request, NULL)`
 #[cfg(not(any(
     target_os = "macos",
     target_os = "ios",
@@ -1587,7 +1629,6 @@ pub(crate) fn clock_nanosleep_absolute(id: ClockId, request: &Timespec) -> io::R
     }
 }
 
-/// `nanosleep(request, remain)`
 #[cfg(not(target_os = "redox"))]
 #[inline]
 #[must_use]
