@@ -8,7 +8,7 @@ mod clock;
 // TODO: Convert WASI'S clock APIs to use handles rather than ambient
 // clock identifiers, update `wasi-libc`, and then add support in `posish`.
 #[cfg(not(any(target_os = "wasi", target_os = "redox")))]
-pub use clock::{clock_getres, clock_gettime, ClockId};
+pub use clock::{clock_getres, clock_gettime, clock_gettime_dynamic, ClockId, DynamicClockId};
 #[cfg(not(target_os = "redox"))]
 pub use clock::{nanosleep, NanosleepRelativeResult};
 
